
#
# Linux-GNU-x86_64 specific settings
#

# Set output dirs
SET_TARGET_PROPERTIES(${PROJECT_NAME} PROPERTIES RUNTIME_OUTPUT_DIRECTORY "${PROJECT_SOURCE_DIR}/Binary/${CMAKE_SYSTEM_NAME}-${CMAKE_SYSTEM_ARCH}/${CMAKE_CXX_COMPILER_ID}/${CMAKE_BUILD_TYPE}")

IF(STREAMRUNTIME_SDK_PROJECT)
  TARGET_LINK_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_EGAC}/Libraries/${CMAKE_SYSTEM_NAME}-${CMAKE_SYSTEM_ARCH}/${CMAKE_CXX_COMPILER_ID}/${CMAKE_BUILD_TYPE}"
    PRIVATE "${PATH_GUILIANI}/Libraries/${CMAKE_SYSTEM_NAME}-${CMAKE_SYSTEM_ARCH}/${CMAKE_CXX_COMPILER_ID}/${CMAKE_BUILD_TYPE}"
  )
ENDIF(STREAMRUNTIME_SDK_PROJECT)

# use cli + shortcuts on this platform
LIST(APPEND PLATFORM_DEFINES "STREAMRUNTIME_USE_CLI;STREAMRUNTIME_USE_SHORTCUTS")

TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
  PRIVATE "${PROJECT_SOURCE_DIR}/Common/Include/Platform/${CMAKE_SYSTEM_NAME}"
  PRIVATE "${PATH_GUILIANI}/Include/Platform/linux"
  PRIVATE "${PATH_GUILIANI}/Include/Platform/POSIX"
  PRIVATE "${PATH_GUILIANI}/Include/Platform/sdl"
  PRIVATE "${PATH_LIBS}/nanosvg/include"
  PRIVATE "/usr/include/freetype2"
)

IF(STREAMRUNTIME_SDK_PROJECT)
  # Some platform depenant include directories.
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_EGAC}/Include/eGML"
    PRIVATE "${PATH_EGAC}/Include/eGML/linux"
    PRIVATE "${PATH_EGAC}/Include/eDBG"
    PRIVATE "${PATH_EGAC}/Include/eC/linux"
    PRIVATE "${PATH_EGAC}/Include/eGaC/linux"
  )
ELSE(STREAMRUNTIME_SDK_PROJECT)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_EGAC}/comps/eGML/code"
    PRIVATE "${PATH_EGAC}/comps/eGML/code/eGML_Linux"
    PRIVATE "${PATH_EGAC}/comps/eDBG/code"
    PRIVATE "${PATH_EGAC}/comps/eC/code/linux"
  )
ENDIF(STREAMRUNTIME_SDK_PROJECT)

IF(GUILIANI_USE_SDL2_PLATFORM) 
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "/usr/include/SDL2"
  )
ELSE(GUILIANI_USE_SDL2_PLATFORM) 
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "/usr/include/SDL"
  )
ENDIF(GUILIANI_USE_SDL2_PLATFORM) 

#eGML
IF(GUILIANI_ADD_EGML_PLATFORM)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_GUILIANI}/Include/Platform/eGML"
  )
ENDIF(GUILIANI_ADD_EGML_PLATFORM)

# ogl
IF("${GUILIANI_OGL_PLATFORM}" STREQUAL "OpenGL")
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_GUILIANI}/Include/Platform/GL"
    PRIVATE "${PATH_GUILIANI}/Include/Platform/GL/OGL"
    PRIVATE "${PATH_LIBS}/opengl/include"
  )
ELSEIF("${GUILIANI_OGL_PLATFORM}" STREQUAL "OpenGL ES 1.1")
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_GUILIANI}/Include/Platform/GL"
    PRIVATE "${PATH_GUILIANI}/Include/Platform/GL/OGLES"
  )
ELSEIF("${GUILIANI_OGL_PLATFORM}" STREQUAL "OpenGL ES 2.0")
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_GUILIANI}/Include/Platform/GL"
    PRIVATE "${PATH_GUILIANI}/Include/Platform/GL/OGLES"
  )
ENDIF("${GUILIANI_OGL_PLATFORM}" STREQUAL "OpenGL")

#DAVE
IF(GUILIANI_ADD_DAVE_PLATFORM)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_GUILIANI}/Include/Platform/DAVE"
    PRIVATE "${PATH_DAVE}/Include"
  )
ENDIF(GUILIANI_ADD_DAVE_PLATFORM)

#BLU
IF(GUILIANI_ADD_BLU_PLATFORM)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_GUILIANI}/Include/Platform/BLU"
    PRIVATE "${PATH_BLU}/Include"
  )
ENDIF(GUILIANI_ADD_BLU_PLATFORM)

#GlyphLib
IF(GUILIANI_ADD_GLYPHLIB_PLATFORM)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_GLYPHLIB}/Include"
  )
ENDIF(GUILIANI_ADD_GLYPHLIB_PLATFORM)

# Set platform depenant include directories
FILE(GLOB PLATFORM_LINUX_PC_CPP
  "${PROJECT_SOURCE_DIR}/Common/Source/Platform/${CMAKE_SYSTEM_NAME}/new_delete.cpp"
  "${PROJECT_SOURCE_DIR}/Common/Source/Platform/${CMAKE_SYSTEM_NAME}/StreamRuntime.cpp"
  "${PROJECT_SOURCE_DIR}/Common/Source/Platform/${CMAKE_SYSTEM_NAME}/StreamRuntimeStartup.cpp"
)
FILE(GLOB PLATFORM_LINUX_PC_H 
  "${PROJECT_SOURCE_DIR}/Common/Include/Platform/${CMAKE_SYSTEM_NAME}/StreamRuntime.h"
  "${PROJECT_SOURCE_DIR}/Common/Include/Platform/${CMAKE_SYSTEM_NAME}/StreamRuntimeStartup.h"
)

SET(PLATFORM_FILES 
  ${PLATFORM_LINUX_PC_CPP}
  ${PLATFORM_LINUX_PC_H}
)
