/*
* Copyright (C) 2004 TES Electronic Solutions GmbH,
* All Rights Reserved.
* This source code and any compilation or derivative thereof is the
* proprietary information of TES Electronic Solutions GmbH
* and is confidential in nature.
* Under no circumstances is this software to be exposed to or placed
* under an Open Source License of any type without the expressed
* written permission of TES Electronic Solutions GmbH
*
*############################################################
*/

#include "StreamRuntimeStartup.h"

#include "GUIConfigDebug.h"

#include "MyGUI_SR.h"
#include "GfxWrap.h"

#include "GUITrace.h"
#include "GUIResourceFileHandler.h"
#include "StreamRuntimeConfig.h"

#include "GUIMemLeakWatcher.h"

int main(int argc, char **argv)
{
    NStreamRuntime::CStreamRuntimeStartup kStreamRuntime;
    eC_String* kArgv = NULL;

    if (argc > 0)
    {
        kArgv = new eC_String[argc];
        for (eC_Int i = 0; i < argc; ++i)
        {
            kArgv[i] = eC_String(argv[i]);
        }
    }

    kStreamRuntime.ProcessCommandLine(argc, kArgv);
    delete[] kArgv;

    kStreamRuntime.LoadLicenseFile();

    try
    {
        // Load configuration (e.g. ScreenSize, initial Dialog etc.) from file
        if (!kStreamRuntime.LoadConfiguration())
        {
            return 0; // Failed to load configuration file.
        }

        // Initialize platform specific wrappers for Graphics, Fonts, Input etc.
        // Use default initialization of wrappers for Windows
        kStreamRuntime.ConstructWrapperClasses();

// only on linux-desktop
#ifdef GFX_USE_GL
        // display license-type in window-caption
        eC_String kText = "Guiliani " + CGUI::GetVersionString() + " - " + CGUI::GetLicenseString();
        GETGFX.SetWindowCaption(kText);
#endif

        // create and run GUI
        kStreamRuntime.CreateGUI();
    }
    catch (...)
    {
        GUILOG(GUI_TRACE_ERROR, "An unrecoverable error occurred. Executing clean shutdown. \n");
    }

    // cleanup wrapper
    kStreamRuntime.DestructWrapperClasses();
    kStreamRuntime.DeInit();

    return(0);
}

