/*
* Copyright (C) 2004 TES Electronic Solutions GmbH,
* All Rights Reserved.
* This source code and any compilation or derivative thereof is the
* proprietary information of TES Electronic Solutions GmbH
* and is confidential in nature.
* Under no circumstances is this software to be exposed to or placed
* under an Open Source License of any type without the expressed
* written permission of TES Electronic Solutions GmbH
*
*############################################################
*/

#include "StreamRuntimeStartup.h"

#include "GUI.h"

#if defined GFX_USE_DIRECTFB
#define INPUT_USE_DEVICE_DIRECTFB
#endif

#ifdef GFX_USE_SDL2
    #include "GfxEnvSDL2.h"
#else
    #include "GfxEnvSDL.h"
#endif

// Graphic and Font Wrapper
#include "FntWrap.h"
#if defined GFX_USE_GL
    #include "GfxWrapOGL.h"
    #include "FntWrapFreeTypeGL.h"
#elif defined GFX_USE_EGL
    #if defined GUILIANI_USE_OGLES11
        #include "GfxWrapOGLES1_1.h"
    #elif defined GUILIANI_USE_OGLES2
        #include "GfxWrapOGLES2.h"
    #else
    #error "undefined OpenGL-version"
    #endif
    #if defined FNT_USE_FREETYPE
        #include "FntWrapFreeTypeGL.h"
    #elif defined FNT_USE_GLYPHLIB
        #include "FntWrapGlyphLibGL.h"
    #endif
#elif defined GFX_USE_EGML
    #include "GfxWrapeGML.h"
    #if defined FNT_USE_FREETYPE
        #include "FntWrapFreeType.h"
    #elif defined FNT_USE_GLYPHLIB
        #include "FntWrapGlyphLib.h"
    #endif
#elif defined GFX_USE_DAVE
    #include "GfxWrapDave.h"
    #if defined FNT_USE_FREETYPE
        #include "FntWrapFreeType.h"
    #elif defined FNT_USE_GLYPHLIB
        #include "FntWrapGlyphLibDave.h"
    #endif
#elif defined GFX_USE_DIRECTFB
    #include "GfxWrapDirectFb.h"
    #include "FntWrapFreeTypeDirectFb.h"
#elif defined GFX_USE_BLU
    #include "GfxWrapBLU.h"
    #if defined FNT_USE_FREETYPE
        #include "FntWrapFreeType.h"
    #elif defined FNT_USE_GLYPHLIB
        #include "FntWrapGlyphLib.h"
    #endif
#else
    #include "GfxWrapNull.h"
#endif

#if defined(GFX_USE_DAVE) || defined(GFX_USE_GL) || defined(GFX_USE_BLU)
#ifdef GUILIANI_ADD_IMAGE_DECODER
    // Image Decoder
    #include "GUIImageLoader.h"
    #include "GUIImageDecoderPNG.h"
    #include "GUIImageDecoderBMP.h"
    #include "GUIImageDecoderJPG.h"
    #include "GUIImageDecoderRAW.h"
    #include "GUIImageDecoderRLE.h"
    #include "GUIImageDecoderSVG.h"
    #include "GUIImageDecoderWEBP.h"
#else
#error "Guiliani is built without Imageloaders. Image will not be loaded correctly."
#endif
#endif

// Sound Wrapper
#if defined SND_USE_NULL
#include "SndWrapNull.h"
#else
#include "SndWrapSDL.h"
#endif

// Input Wrapper
#if defined INPUT_USE_DEVICE_UNIX
    #include "GUIInputDeviceUnix.h"
    #include "GUIInputUnixMouse.h"
#elif defined INPUT_USE_DEVICE_DIRECTFB
    #include "GUIInputDirectFb.h"
#else
#ifdef GFX_USE_SDL2
    #include "GUIInputSDL2.h"
#else
    #include "GUIInputSDL.h"
#endif

#endif
#include "GUINullInput.h"
#include "GUIPlaybackInput.h"

#include "GUIException.h"

#include "GUIMemLeakWatcher.h"

// StreamRuntimeStartup_Linux.cpp : Linux specific default startup code for a Guiliani StreamRuntime
namespace NStreamRuntime
{
    void CStreamRuntimeStartup::ConstructWrapperClasses()
    {
        GUILOG( GUI_TRACE_DEBUG, "CStreamRuntimeStartupLinux::ConstructWrapperClasses \n");

        // Sets the GraphicsWrapper and corresponding FontWrapper
        try
        {
#if defined GFX_USE_GL
    #if defined GFX_USE_SDL2
        CGfxEnvSDL2::CreateInstance(CGfxEnv::SM_OPENGL, CGUICommonEnums::ARGB_8888, GetScreenWidth(), GetScreenHeight(), IsWindowResizable());
    #else
        CGfxEnvSDL::CreateInstance(CGfxEnv::SM_OPENGL, CGUICommonEnums::ARGB_8888, GetScreenWidth(), GetScreenHeight(), IsWindowResizable());
    #endif
#else
    #if defined GFX_USE_SDL2
        CGfxEnvSDL2::CreateInstance(CGfxEnv::SM_FRAMEBUFFER, CGUICommonEnums::ARGB_8888, GetScreenWidth(), GetScreenHeight(), IsWindowResizable());
    #else
        CGfxEnvSDL::CreateInstance(CGfxEnv::SM_FRAMEBUFFER, CGUICommonEnums::ARGB_8888, GetScreenWidth(), GetScreenHeight(), IsWindowResizable());
    #endif
#endif

#if 1
        // initialize layers from configuration
        const eC_UInt uiLayerCount = GETRUNTIMECONFIG.GetLayerCount();
        for (eC_UInt uiLayerIndex = 0; uiLayerIndex < uiLayerCount; ++uiLayerIndex)
        {
            LayerConfiguration_t rkLayerConfig = GETRUNTIMECONFIG.GetLayerConfig(uiLayerIndex);
            //rkLayerConfig.m_uiNumBuffers = 2;
            //eC_UByte* pkBuffer0 = new eC_UByte[rkLayerConfig.m_uiWidth * rkLayerConfig.m_uiHeight * 4];
            //eC_UByte* pkBuffer1 = new eC_UByte[rkLayerConfig.m_uiWidth * rkLayerConfig.m_uiHeight * 4];
            //rkLayerConfig.m_pkBuffers[0] = pkBuffer0;
            //rkLayerConfig.m_pkBuffers[1] = pkBuffer1;
            GETGFXENV.AddLayer(rkLayerConfig);
        }
#endif
        GETGFXENV.InitLayers();

// opengl
#if defined GFX_USE_GL
        CGfxWrapOGL::CreateInstance(&GETGFXENV);
// freetype or glyphlib
    #if defined FNT_USE_FREETYPE
        CFntWrapFreeTypeGL::CreateInstance();
    #elif defined FNT_USE_GLYPHLIB
        CFntWrapGlyphLibGL::CreateInstance();
    #endif
// opengl es
#elif defined GFX_USE_EGL
    #if defined GUILIANI_USE_OGLES11
        CGfxWrapOGLES1_1::CreateInstance(&nativewindow, EGL_DEFAULT_DISPLAY);
    #elif defined GUILIANI_USE_OGLES2
        CGfxWrapOGLES2::CreateInstance(&nativewindow, EGL_DEFAULT_DISPLAY);
    #else
    #error "undefined OpenGL-version"
    #endif
        CFntWrapFreeTypeGL::CreateInstance();
// directfb
#elif defined GFX_USE_DIRECTFB
        CGfxWrapDirectFb::CreateInstance(GetScreenWidth(), GetScreenHeight(), false, CGfxWrapDirectFb::BPP_FORMAT16);
        CFntWrapFreeTypeDirectFb::CreateInstance(1024*1024,(CGfxWrapDirectFb&)CGfxWrapDirectFb::GetInstance());
// egml
#elif defined GFX_USE_EGML
        // base-config is used from main-layer
        CGfxWrapeGML::CreateInstance(&GETGFXENV);

    #if defined FNT_USE_FREETYPE
            CFntWrapFreeType::CreateInstance(1024 * 1024);
    #elif defined FNT_USE_GLYPHLIB
            CFntWrapGlyphLib::CreateInstance();
    #endif
// blu
#elif defined GFX_USE_BLU
        CGfxWrapBLU::CreateInstance(&GETGFXENV);

#if defined FNT_USE_FREETYPE
        CFntWrapFreeType::CreateInstance(1024 * 1024);
#elif defined FNT_USE_GLYPHLIB
        CFntWrapGlyphLib::CreateInstance();
#endif

// dave
#elif defined GFX_USE_DAVE
        // base-config is used from main-layer
        CGfxWrapDAVE::CreateInstance(&GETGFXENV);

#if defined FNT_USE_FREETYPE
        CFntWrapFreeType::CreateInstance(1024 * 1024);
#elif defined FNT_USE_GLYPHLIB
        CFntWrapGlyphLib::CreateInstance();
#endif
#elif defined GFX_USE_EGL

#else
#error "Selected GfxWrapper not available!"
#endif
        }
        catch(...)
        {
            GUILOG_THROW_EXCEPTION(CGUIException(), "CMyGui::Init: Could not allocate graphics wrapper.\n");
        }

        // Sets the SoundWrapper
        try
        {
#if defined SND_USE_NULL
            CSndWrapNull::CreateInstance();
#else
            CSndWrapSDL::CreateInstance();
#endif
        }
        catch (...)
        {
            GUILOG_THROW_EXCEPTION(CGUIException(), "CMyGui::Init: Could not allocate sound wrapper. \n");
        }

        try
        {
#if defined INPUT_USE_DEVICE_UNIX
            eC_Char *pkKeyboard = GetKeyboardDevice().ToASCII_Alloc();
            eC_Char *pkTouch = GetTouchDevice().ToASCII_Alloc();
            eC_Char *pkRemote = GetRemoteDevice().ToASCII_Alloc();
            CGUIInputDeviceUnix::CreateInstance(pkKeyboard, pkTouch, pkRemote, GetTouchOffsetX(), GetTouchOffsetY(), GetTouchSizeX(), GetTouchSizeY(), 2);

            delete[] pkKeyboard;
            delete[] pkTouch;
            delete[] pkRemote;
#elif defined INPUT_USE_DEVICE_DIRECTFB
            CGUIInputDirectFb::CreateInstance();
#else
#ifdef GFX_USE_SDL2
            CGUIInputSDL2::CreateInstance();
#else
            CGUIInputSDL::CreateInstance();
#endif
#endif
        }
        catch(...)
        {
            GUILOG_THROW_EXCEPTION(CGUIException(), "CMyGUI::Init: Could not create input media.\n");
        }

#if defined(GFX_USE_DAVE) || defined(GFX_USE_GL) || defined(GFX_USE_BLU)
#ifdef GUILIANI_ADD_IMAGE_DECODER
        // add image-decoders
        GETIMGLOADER.AddDecoder(new CGUIImageDecoderBMP());
        GETIMGLOADER.AddDecoder(new CGUIImageDecoderPNG());
        GETIMGLOADER.AddDecoder(new CGUIImageDecoderJPG());
        GETIMGLOADER.AddDecoder(new CGUIImageDecoderRAW());
        GETIMGLOADER.AddDecoder(new CGUIImageDecoderRLE());
        GETIMGLOADER.AddDecoder(new CGUIImageDecoderSVG());
        GETIMGLOADER.AddDecoder(new CGUIImageDecoderWEBP());
#endif
#endif
    }
}
