/*
* Copyright (C) 2004 TES Electronic Solutions GmbH,
* All Rights Reserved.
* This source code and any compilation or derivative thereof is the
* proprietary information of TES Electronic Solutions GmbH
* and is confidential in nature.
* Under no circumstances is this software to be exposed to or placed
* under an Open Source License of any type without the expressed
* written permission of TES Electronic Solutions GmbH
*
*############################################################
*/

#include "StreamRuntimeStartup.h"

#include "GUITrace.h"

#if defined GFX_USE_EGML
    #include "GfxEnvFBDevice.h"
    #include "GfxWrapeGML.h"
    #if defined FNT_USE_FREETYPE
        #include "FntWrapFreeType.h"
    #elif defined FNT_USE_GLYPHLIB
        #include "FntWrapGlyphLib.h"
    #endif
#else
    #error "unsupported GfxWrap requested"
#endif

#ifdef GUILIANI_ADD_IMAGE_DECODER
    // Image Decoder
    #include "GUIImageLoader.h"
    #include "GUIImageDecoderBMP.h"
    #include "GUIImageDecoderPNG.h"
    #include "GUIImageDecoderJPG.h"
    #include "GUIImageDecoderSVG.h"
    #include "GUIImageDecoderRLE.h"
    #include "GUIImageDecoderRAW.h"
#endif

// Sound Wrapper
#include "SndWrapNull.h"

// Input Wrapper
#include "GUIInputDeviceUnix.h"
#include "GUIInputUnixMouse.h"

#include "GUIException.h"

#include "GUIMemLeakWatcher.h"

namespace NStreamRuntime
{
    void CStreamRuntimeStartup::ConstructWrapperClasses()
    {
        GUILOGMESSAGE( "StreamRuntimeStartup.cpp NStreamRuntime::ConstructResourceClasses \n");
        // Sets the GraphicsWrapper
        try
        {
#if defined GFX_USE_EGML
            CGfxEnvFBDevice::CreateInstance(GetScreenWidth(), GetScreenHeight(), 1, IsWindowResizable());
            CGfxWrapeGML::CreateInstance(&GETGFXENV);
            static_cast<CGfxWrapeGML&>(GETGFX).SetSubpixelEnabled(false);
    #if defined FNT_USE_FREETYPE
            //create a fontwrapper for freetype with 1024 kByte cache.
            CFntWrapFreeType::CreateInstance(1024*1024);
    #elif defined FNT_USE_GLYPHLIB
            CFntWrapGlyphLib::CreateInstance();
    #endif
#endif

#ifdef GUILIANI_ADD_IMAGE_DECODER
            // add image-decoders
            GETIMGLOADER.AddDecoder(new CGUIImageDecoderBMP());
            GETIMGLOADER.AddDecoder(new CGUIImageDecoderPNG());
            GETIMGLOADER.AddDecoder(new CGUIImageDecoderJPG());
            GETIMGLOADER.AddDecoder(new CGUIImageDecoderSVG());
            GETIMGLOADER.AddDecoder(new CGUIImageDecoderRLE());
            GETIMGLOADER.AddDecoder(new CGUIImageDecoderRAW());
#endif

        }
        catch(...)
        {
            GUILOG_THROW_EXCEPTION(CGUIException(), "CStreamRuntimeStartup::ConstructWrapperClasses: Could not allocate graphics wrapper.\n");
        }

        // Sets the SoundWrapper
        try
        {
            CSndWrapNull::CreateInstance();
        }
        catch (...)
        {
            GUILOG_THROW_EXCEPTION(CGUIException(), "CStreamRuntimeStartup::ConstructWrapperClasses: Could not allocate sound wrapper.\n");
        }

        if (!GetMouseDevice().IsEmpty() && !GetTouchDevice().IsEmpty())
        {
            GUILOG_THROW_EXCEPTION(CGUIException(), "CStreamRuntimeStartup::ConstructWrapperClasses: two different input-types provided.\n");        
        }

        try
        {
            if (!GetMouseDevice().IsEmpty())
            {
                eC_Char* pkMouse = GetMouseDevice().ToASCII_Alloc();
                CGUIInputUnixMouse::CreateInstance(
                    pkMouse,
                    GETGFX.GetPhysicalScreenWidth(),
                    GETGFX.GetPhysicalScreenHeight());
                delete[] pkMouse;
            }
            else if (!GetTouchDevice().IsEmpty())
            {
                eC_Char* pkKeyboard = GetKeyboardDevice().ToASCII_Alloc();
                eC_Char* pkTouch = GetTouchDevice().ToASCII_Alloc();
                eC_Char* pkRemote = GetRemoteDevice().ToASCII_Alloc();
                CGUIInputDeviceUnix::CreateInstance(
                    pkKeyboard, pkTouch, pkRemote,
                    GetTouchOffsetX(), GetTouchOffsetY(),
                    GetTouchSizeX(), GetTouchSizeY(), 2);

                delete[] pkKeyboard;
                delete[] pkTouch;
                delete[] pkRemote;
            }
            else
            {
                GUILOG_THROW_EXCEPTION(CGUIException(), "CStreamRuntimeStartup::ConstructWrapperClasses: No input device given.\n");
            }
        }
        catch(...)
        {
            GUILOG_THROW_EXCEPTION(CGUIException(), "CStreamRuntimeStartup::ConstructWrapperClasses: Could not create input media.\n");
        }
    }
}
